// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __colorbarbase_h__
#define __colorbarbase_h__

#include "widget.h"

// ColorbarBase

class CBGrid;

struct ColorbarBaseOptions {
  Tk_Item item;              // required by tk
  int x, y;                  // Coordinates of positioning point on canvas
  int width;                 // widget width
  int height;                // widget height
  Tk_Anchor anchor;          // Where to anchor widget relative to x,y
  char* cmdName;             // Suggested Tcl command name

  Widget* widget;            // pointer to widget class

  int numheight;
  char* numopts;

  int colors;
  int minColors;
  int maxColors;
  int privateCmap;
  int privateColors;
};

class ColorbarBase : public Widget {
  friend class CBGrid;

 private:
  void updateGCs();

 protected:
  XImage* xmap;
  unsigned char* colorCells;
  int colorCount;

  int numerics;
  CBGrid* grid;
  GC gc;

  int cnt;
  double* lut;

  int invert;

 protected:
  virtual int initColormap() =0;
  virtual void updateColors() =0;
  int updatePixmap(const BBox&);
  virtual void ximageToPixmap() =0;
  void invalidPixmap();
  virtual void reset() =0;

  int postscriptProc(int);   // generate postscript
  virtual void psLevel1(PSColorSpace, int, int, float) =0;
  virtual void psLevel2(PSColorSpace, int, int, float) =0;

#ifdef _MACOSX
  virtual void macosx(float, int, int, const Vector&, const Vector&) =0;
#endif
#ifdef _WIN32
  virtual void win32(float, int, int, const Vector&, const Vector&) =0;
#endif

 public:
  ColorbarBase(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~ColorbarBase();

  int parse(istringstream&);
  int configure(int, const char* [], int);

  virtual void adjustCmd(float, float) =0;
  virtual void getBiasCmd() =0;
  virtual void getColormapCmd() =0;
  virtual void getColormapNameCmd(int) =0;
  virtual void getContrastCmd() =0;
  virtual void getCurrentFileNameCmd() =0;
  virtual void getCurrentIDCmd() =0;
  virtual void getCurrentNameCmd() =0;

  void getNumericsCmd();
  void numericsCmd(int);

  void setColormapLevelCmd(int, float*);
  void getInvertCmd();
  void invertCmd(int);

#ifdef _MACOSX
  void macosxPrintCmd();
#endif
#ifdef _WIN32
  void win32PrintCmd();
#endif
};

#endif

