// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include <stdlib.h>
#include <limits.h>

#include "tcl.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "colorbarrgbtrue.h"
#include "util.h"
#include "ps.h"

ColorbarRGBTrueColor::ColorbarRGBTrueColor(Tcl_Interp* i, Tk_Canvas c,
					   Tk_Item* item) 
  : ColorbarRGB(i, c, item)
{
  colorCount = 0;
}

int ColorbarRGBTrueColor::initColormapForReal()
{
  colorCells = new unsigned char[colorCount*3];

 // needed to initialize colorCells
  reset();

  return TCL_OK;
}

void ColorbarRGBTrueColor::updateColorCells()
{
  // fill rgb table
  // note: its filled bgr to match XImage
  //  for(int i=0; i<colorCount; i++) {
  for(int i=0, j=colorCount-1; i<colorCount; i++, j--) {
    int idr = invert ? calcContrastBias(j,bias[0],contrast[0]) : 
      calcContrastBias(i,bias[0],contrast[0]);
    int idg = invert ? calcContrastBias(j,bias[1],contrast[1]) : 
      calcContrastBias(i,bias[1],contrast[1]);
    int idb = invert ? calcContrastBias(j,bias[2],contrast[2]) : 
      calcContrastBias(i,bias[2],contrast[2]);

    colorCells[i*3]   = (int)(256.*idr/colorCount);
    colorCells[i*3+1] = (int)(256.*idg/colorCount);
    colorCells[i*3+2] = (int)(256.*idb/colorCount);
  }
}

void ColorbarRGBTrueColor::psLevel1(PSColorSpace mode, int width, int height, 
				    float scale)
{
  if (!colorCells)
    return;

  psLevel1Head(mode, width, height);
  NoCompressAsciiHex filter;

  // red
  for (int j=0; j<(int)(height/3.); j++) {
    ostringstream str;

    for (int i=0; i<width; i++) {
      unsigned char red = colorCells[(int)(double(i)/width*colorCount)*3+2];
      unsigned char green = 0;
      unsigned char blue = 0;

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
      case CMYK:
	filter << red << green << blue;
	break;
      }
      str << filter;
    }

    str << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  // green
  for (int j=(int)(height/3.); j<(int)(height*2/3.); j++) {
    ostringstream str;

    for (int i=0; i<width; i++) {
      unsigned char red = 0;
      unsigned char green = colorCells[(int)(double(i)/width*colorCount)*3+1];
      unsigned char blue = 0;

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
      case CMYK:
	filter << red << green << blue;
	break;
      }
      str << filter;
    }

    str << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  // blue
  for (int j=(int)(height*2/3.); j<height; j++) {
    ostringstream str;

    for (int i=0; i<width; i++) {
      unsigned char red = 0;
      unsigned char green = 0;
      unsigned char blue = colorCells[(int)(double(i)/width*colorCount)*3];

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
      case CMYK:
	filter << red << green << blue;
	break;
      }
      str << filter;
    }

    str << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  ostringstream str;
  filter.flush(str);
  Tcl_AppendResult(interp, str.str().c_str(), NULL);
}

void ColorbarRGBTrueColor::psLevel2(PSColorSpace mode, int width, int height, 
				    float scale)
{
  if (!colorCells)
    return;

  psLevel2HeadTrue(mode, width, height);
  RLEAscii85 filter;

  // red
  for (int j=0; j<(int)(height/3.); j++) {
    ostringstream str;

    for (int i=0; i<width; i++) {
      unsigned char red = colorCells[(int)(double(i)/width*colorCount)*3+2];
      unsigned char green = 0;
      unsigned char blue = 0;

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
	filter << red << green << blue;
	break;
      case CMYK:
	{
	  unsigned char cyan, magenta, yellow, black;
	  RGB2CMYK(red, green, blue, &cyan, &magenta, &yellow, &black);
	  filter << cyan << magenta << yellow << black;
	}
	break;
      }
      str << filter;
    }

    str << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  // green
  for (int j=(int)(height/3.); j<(int)(height*2/3.); j++) {
    ostringstream str;

    for (int i=0; i<width; i++) {
      unsigned char red = 0;
      unsigned char green = colorCells[(int)(double(i)/width*colorCount)*3+1];
      unsigned char blue = 0;

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
	filter << red << green << blue;
	break;
      case CMYK:
	{
	  unsigned char cyan, magenta, yellow, black;
	  RGB2CMYK(red, green, blue, &cyan, &magenta, &yellow, &black);
	  filter << cyan << magenta << yellow << black;
	}
	break;
      }
      str << filter;
    }

    str << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  // blue
  for (int j=(int)(height*2/3.); j<height; j++) {
    ostringstream str;

    for (int i=0; i<width; i++) {
      unsigned char red = 0;
      unsigned char green = 0;
      unsigned char blue = colorCells[(int)(double(i)/width*colorCount)*3];

      switch (mode) {
      case BW:
      case GRAY:
	filter << RGB2Gray(red, green, blue);
	break;
      case RGB:
	filter << red << green << blue;
	break;
      case CMYK:
	{
	  unsigned char cyan, magenta, yellow, black;
	  RGB2CMYK(red, green, blue, &cyan, &magenta, &yellow, &black);
	  filter << cyan << magenta << yellow << black;
	}
	break;
      }
      str << filter;
    }

    str << ends;
    Tcl_AppendResult(interp, str.str().c_str(), NULL);
  }

  ostringstream str;
  filter.flush(str);
  psFix(str);
  Tcl_AppendResult(interp, str.str().c_str(), NULL);
}

#ifdef _MACOSX
void ColorbarRGBTrueColor::macosx(float scale, int width, int height, 
				  const Vector& v, const Vector& s)
{
  if (!colorCells)
    return;

  // destination
  unsigned char* dst = new unsigned char[width*height*4];
  unsigned char* dptr = dst;

  // flipped in Y
  // blue
  for (int jj=0; jj<(int)(height/3.); jj++)
    for (int ii=0; ii<width; ii++) {
      *dptr++ = 0;
      *dptr++ = 0;
      *dptr++ = colorCells[(int)(double(ii)/width*colorCount)*3];
      *dptr++ = 0;
    }

  // green
  for (int jj=(int)(height/3.); jj<(int)(height*2/3.); jj++)
    for (int ii=0; ii<width; ii++) {
      *dptr++ = 0;
      *dptr++ = colorCells[(int)(double(ii)/width*colorCount)*3+1];
      *dptr++ = 0;
      *dptr++ = 0;
    }

  // red
  for (int jj=(int)(height*2/3.); jj<height; jj++) 
    for (int ii=0; ii<width; ii++) {
      *dptr++ = colorCells[(int)(double(ii)/width*colorCount)*3+2];
      *dptr++ = 0;
      *dptr++ = 0;
      *dptr++ = 0;
    }

  macosxBitmapCreate(dst, width, height, v, s);

  if (dst)
    delete dst;
}
#endif

#ifdef _WIN32
void ColorbarRGBTrueColor::win32(float scale, int width, int height, 
				  const Vector& v, const Vector& s)
{
  if (!colorCells)
    return;

  // why is this needed?
  width++;
  // height++;

  // destination
  unsigned char* dst = new unsigned char[width*height*3];
  unsigned char* dptr = dst;

  // flipped in Y
  // blue
  for (int jj=0; jj<(int)(height/3.); jj++)
    for (int ii=0; ii<width; ii++) {
      *dptr++ = colorCells[(int)(double(ii)/width*colorCount)*3];
      *dptr++ = 0;
      *dptr++ = 0;
    }

  // green
  for (int jj=(int)(height/3.); jj<(int)(height*2/3.); jj++)
    for (int ii=0; ii<width; ii++) {
      *dptr++ = 0;
      *dptr++ = colorCells[(int)(double(ii)/width*colorCount)*3+1];
      *dptr++ = 0;
    }

  // red
  for (int jj=(int)(height*2/3.); jj<height; jj++) 
    for (int ii=0; ii<width; ii++) {
      *dptr++ = 0;
      *dptr++ = 0;
      *dptr++ = colorCells[(int)(double(ii)/width*colorCount)*3+2];
    }

  win32BitmapCreate(dst, width, height, v, s);

  if (dst)
    delete dst;
}
#endif
