// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include <stdlib.h>
#include <limits.h>
#include <string.h>

#include "colorbarrgbtrue16.h"
#include "util.h"

// Tk Canvas Widget Function Declarations

int ColorbarRGBTrueColor16CreateProc(Tcl_Interp*, Tk_Canvas, Tk_Item*, int, 
				  Tcl_Obj *const []);

// ColorbarRGB Specs

static Tk_CustomOption tagsOption = {
  Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
};

static Tk_ConfigSpec colorbarRGBTrueColor16Specs[] = {

  {TK_CONFIG_STRING, "-command", NULL, NULL, "colorbarrgb",
   Tk_Offset(ColorbarBaseOptions, cmdName), 
   TK_CONFIG_OPTION_SPECIFIED,NULL},
  {TK_CONFIG_INT, "-x", NULL, NULL, "1",
   Tk_Offset(ColorbarBaseOptions, x), 
   TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-y", NULL, NULL, "1",
   Tk_Offset(ColorbarBaseOptions, y), 
   TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-width", NULL, NULL, "512",
   Tk_Offset(ColorbarBaseOptions, width), 
   TK_CONFIG_OPTION_SPECIFIED, 
   NULL},
  {TK_CONFIG_INT, "-height", NULL, NULL, "22",
   Tk_Offset(ColorbarBaseOptions, height), 
   TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_ANCHOR, "-anchor", NULL, NULL, "nw",
   Tk_Offset(ColorbarBaseOptions, anchor), 0, NULL},
  {TK_CONFIG_CUSTOM, "-tags", NULL, NULL, NULL,
   0, TK_CONFIG_NULL_OK, &tagsOption},

  {TK_CONFIG_INT, "-numheight", NULL, NULL, "0",
   Tk_Offset(ColorbarBaseOptions, numheight), 0, NULL},
  {TK_CONFIG_STRING, "-numopts", NULL, NULL, "",
   Tk_Offset(ColorbarBaseOptions, numopts), 0, NULL},

  {TK_CONFIG_INT, "-colors", NULL, NULL, "1024",
   Tk_Offset(ColorbarBaseOptions, colors), 0, NULL},

  {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL},
};

// Tk Static Structure

static Tk_ItemType colorbarRGBTrueColor16Type = {
  "colorbarrgbtruecolor16",        // name
  sizeof(ColorbarBaseOptions), // size
  ColorbarRGBTrueColor16CreateProc, // configProc
  colorbarRGBTrueColor16Specs,     // configSpecs
  WidgetConfigProc,             // configProc
  WidgetCoordProc,              // coordProc
  WidgetDeleteProc,             // deleteProc
  WidgetDisplayProc,            // displayProc
  0,                            // alwaysRedraw
  WidgetPointProc,              // pointProc
  WidgetAreaProc,               // areaProc
  WidgetPostscriptProc,         // postscriptProc
  WidgetScaleProc,              // scaleProc
  WidgetTranslateProc,          // translateProc
  (Tk_ItemIndexProc*)NULL,      // indexProc
  (Tk_ItemCursorProc*)NULL,     // icursorProc
  (Tk_ItemSelectionProc*)NULL,  // selectionProc
  (Tk_ItemInsertProc*)NULL,     // insertProc
  (Tk_ItemDCharsProc*)NULL,     // dCharsProc
  (Tk_ItemType*)NULL            // nextPtr
};

// Non-Member Functions

int ColorbarRGBTrueColor16_Init(Tcl_Interp* interp)
{
  Tk_CreateItemType(&colorbarRGBTrueColor16Type);
  return TCL_OK;
}

int ColorbarRGBTrueColor16CreateProc(Tcl_Interp* interp, Tk_Canvas canvas, 
				     Tk_Item* item, int argc, 
				     Tcl_Obj *const argv[])
{
  ColorbarRGBTrueColor16* colorbar = 
    new ColorbarRGBTrueColor16(interp, canvas, item);

  // and set default configuration
  if (colorbar->configure(argc, (const char**)argv, 0) != TCL_OK) {
    delete colorbar;
    Tcl_AppendResult(interp, " error occured while creating colorbar.", NULL);
    return TCL_ERROR;
  }

  return TCL_OK;
}

// ColorbarRGBTrueColor16

ColorbarRGBTrueColor16::ColorbarRGBTrueColor16(Tcl_Interp* i, Tk_Canvas c, 
					       Tk_Item* item) 
  : ColorbarRGBTrueColor(i, c, item)
{
  configSpecs = colorbarRGBTrueColor16Specs;  // colorbar configure options
}

int ColorbarRGBTrueColor16::initColormap()
{
  colorCount = (((ColorbarBaseOptions*)options)->colors);
  return initColormapForReal();
}

void ColorbarRGBTrueColor16::updateColors()
{
  updateColorCells();

  // fill in xmap
  // make sure we have a pixmap
  if (!pixmap || !xmap)
    return;

  int width = options->width-2;
  int height = options->height - 
    ((ColorbarBaseOptions*)options)->numheight -2;
  char* data = XImageData(xmap);
    
  // Fill in colorbar data
  // Calculate first row
  // check for bits/pixel

  int rs,gs,bs;
#if !(_WIN32 || _MACOSX)
  unsigned short rm = decodeMask((unsigned short)visual->red_mask, &rs);
  unsigned short gm = decodeMask((unsigned short)visual->green_mask, &gs);
  unsigned short bm = decodeMask((unsigned short)visual->blue_mask, &bs);
#endif
#if _MACOSX
  // tk visual masks are bogus
  unsigned short rm = decodeMask((unsigned short)xmap->red_mask, &rs);
  unsigned short gm = decodeMask((unsigned short)xmap->green_mask, &gs);
  unsigned short bm = decodeMask((unsigned short)xmap->blue_mask, &bs);
#endif
#if _WIN32
  // windows masks are plain wrong
  unsigned short rm = decodeMask((unsigned short)0x7C00, &rs);
  unsigned short gm = decodeMask((unsigned short)0x03E0, &gs);
  unsigned short bm = decodeMask((unsigned short)0x001F, &bs);
#endif

  // if we have cross platforms, we need to byte swap

  unsigned char row[xmap->bytes_per_line];
  if ((!xmap->byte_order && lsb()) || (xmap->byte_order && !lsb())) {
    for (int i=0; i<width; i++) {
      int j = ((int)(double(i)/width*colorCount))*3;
      unsigned short r = colorCells[j];
      unsigned short a = 0;
      a |= rs>0 ? ((r & rm) << rs) : ((r & rm) >> -rs);
      memcpy(row+i*2, &a, 2);
    }
    for (int j=0; j<(int)(height/3.); j++)
      memcpy(data+(j*xmap->bytes_per_line), row, xmap->bytes_per_line);

    for (int i=0; i<width; i++) {
      int j = ((int)(double(i)/width*colorCount))*3;
      unsigned short g = colorCells[j+1];
      unsigned short a = 0;
      a |= gs>0 ? ((g & gm) << gs) : ((g & gm) >> -gs);
      memcpy(row+i*2, &a, 2);
    }
    for (int j=(int)(height/3.); j<(int)(height*2/3.); j++)
      memcpy(data+(j*xmap->bytes_per_line), row, xmap->bytes_per_line);

    for (int i=0; i<width; i++) {
      int j = ((int)(double(i)/width*colorCount))*3;
      unsigned short b = colorCells[j+2];
      unsigned short a = 0;
      a |= bs>0 ? ((b & bm) << bs) : ((b & bm) >> -bs);
      memcpy(row+i*2, &a, 2);
    }
    for (int j=(int)(height*2/3.); j<height; j++)
      memcpy(data+(j*xmap->bytes_per_line), row, xmap->bytes_per_line);

  }
  else {
    for (int i=0; i<width; i++) {
      int j = ((int)(double(i)/width*colorCount))*3;
      unsigned short r = colorCells[j];
      unsigned short a = 0;
      a |= rs>0 ? ((r & rm) << rs) : ((r & rm) >> -rs);
      unsigned char* rr = (unsigned char*)(&a);
      *(row+i*2) = *(rr+1);
      *(row+i*2+1) = *(rr);
    }
    for (int j=0; j<(int)(height/3.); j++)
      memcpy(data+(j*xmap->bytes_per_line), row, xmap->bytes_per_line);

    for (int i=0; i<width; i++) {
      int j = ((int)(double(i)/width*colorCount))*3;
      unsigned short g = colorCells[j+1];
      unsigned short a = 0;
      a |= gs>0 ? ((g & gm) << gs) : ((g & gm) >> -gs);
      unsigned char* rr = (unsigned char*)(&a);
      *(row+i*2) = *(rr+1);
      *(row+i*2+1) = *(rr);
    }
    for (int j=(int)(height/3.); j<(int)(height*2/3.); j++)
      memcpy(data+(j*xmap->bytes_per_line), row, xmap->bytes_per_line);

    for (int i=0; i<width; i++) {
      int j = ((int)(double(i)/width*colorCount))*3;
      unsigned short b = colorCells[j+2];
      unsigned short a = 0;
      a |= bs>0 ? ((b & bm) << bs) : ((b & bm) >> -bs);
      unsigned char* rr = (unsigned char*)(&a);
      *(row+i*2) = *(rr+1);
      *(row+i*2+1) = *(rr);
    }
    for (int j=(int)(height*2/3.); j<height; j++)
      memcpy(data+(j*xmap->bytes_per_line), row, xmap->bytes_per_line);
  }

  // update the pixmap from map image and clean up
  XPutImage(display, pixmap, gc, xmap, 0, 0, 1, 1, width, height);

  redraw();
}

