// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __fitshist_h__
#define __fitshist_h__

#include "vector.h"
#include "file.h"

class FitsHist : public FitsFile {
 public:
  enum Function {SUM, AVERAGE};

 private:
  int width;
  int height;
  int depth;
  off_t size;

  FitsColumn* xcol;
  FitsColumn* ycol;
  FitsColumn* zcol;

  void* fitsy;
  void* filter;

  int initHeader(FitsFile*, Matrix&);
  void initLTMV(Matrix&);
  void initWCS(FitsFile*, Matrix&, Vector);
  void mapWCSString(FitsHead*, char* w, char* out, char* prim, char* alt);
  void mapWCSReal(FitsHead*, char* w, char* out, char* prim, char* alt, Matrix);
  void mapWCSReal(FitsHead* head, char* w, char* out, char* in);
  void mapWCSMatrix(FitsHead*, char* w, char* out, char* in, Vector);
  void mapWCSVector(FitsHead*, char* w, char* out, char* in);
  void initFilter(FitsFile*);
  void deleteFilter();
  void bin(FitsFile*, Matrix&, Function, Vector);
  void swap();

 public:
  FitsHist(FitsFile* src, int width, int height, int depth, 
	   Matrix& m, Function func, Vector block);
  ~FitsHist();
};

class FitsHistNext : public FitsFile {
public:
  FitsHistNext(FitsFile* prev);
};

#endif
