// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "colorscalergb.h"
#include "util.h"

ColorScaleRGB::ColorScaleRGB(int s)
{
  size_ = s;
  psColors_ = new unsigned char[s];

  memset(psColors_, '0', size_);
}

ColorScaleRGB::~ColorScaleRGB()
{
  if (psColors_)
    delete [] psColors_;
}

LinearScaleRGB::LinearScaleRGB(int ii, int s, unsigned char* colorCells, 
			       int count) 
  : ColorScaleRGB(s)
{
  for (int i=0; i<s; i++) {
    double a = double(i)/s;
    int l = (int)(a * count);
    memcpy(psColors_+i, colorCells+l*3+ii,1);
  }
}

LogScaleRGB::LogScaleRGB(int ii, int s, unsigned char* colorCells, int count,
			 double low, double high) : ColorScaleRGB(s)
{
  CheckScaleLog(&low,&high);

  for (int i=0; i<s; i++) {
    double a = log10(high/low*double(i)/s +1) / log10(high/low +1);
    int l = (int)(a * count);
    memcpy(psColors_+i, colorCells+l*3+ii,1);
  }
}

PowScaleRGB::PowScaleRGB(int ii, int s, unsigned char* colorCells, int count,
			 double low, double high) : ColorScaleRGB(s)
{
  CheckScaleLog(&low,&high);

  for (int i=0; i<s; i++) {
    double a = ::pow(high/low, double(i)/s) *low/high;
    //special case
    if (a==1)
      a = 0;
    int l = (int)(a * count);
    memcpy(psColors_+i, colorCells+l*3+ii,1);
  }
}

SqrtScaleRGB::SqrtScaleRGB(int ii, int s, unsigned char* colorCells, int count)
  : ColorScaleRGB(s)
{
  for(int i=0; i<s; i++) {
    double a = double(i)/s;
    int l = (int)(sqrt(a) * count);
    memcpy(psColors_+i, colorCells+l*3+ii,1);
  }
}

SquaredScaleRGB::SquaredScaleRGB(int ii, int s, unsigned char* colorCells, 
				 int count) 
  : ColorScaleRGB(s)
{
  for(int i=0; i<s; i++) {
    double a = double(i)/s;
    int l = (int)(a*a * count);
    memcpy(psColors_+i, colorCells+l*3+ii,1);
  }
}

HistEquScaleRGB::HistEquScaleRGB(int ii, int s, unsigned char* colorCells, 
				 int count, double* hist, int histsize)
  : ColorScaleRGB(s)
{
  // if no histogram, return linear distribution
  if (!hist)
    for (int i=0; i<s; i++) {
      double a = double(i)/s;
      int l = (int)(a * count);
      memcpy(psColors_+i, colorCells+l*3+ii,1);
    }
  else
    for (int i=0; i<s; i++) {
      double a = hist[i*histsize/s];
      int l = (int)(a * count);
      memcpy(psColors_+i, colorCells+l*3+ii,1);
    }
}
