// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "colorscaletrue32.h"
#include "util.h"

ColorScaleTrueColor32::ColorScaleTrueColor32(int s, 
					     unsigned long red_mask,
					     unsigned long green_mask,
					     unsigned long blue_mask,
					     int msb) 
  : ColorScale(s)
{
  // this makes the assumption of one byte per color.

  colors_ = new unsigned char[s*4];
  int rs = decodeMask(red_mask);
  int gs = decodeMask(green_mask);
  int bs = decodeMask(blue_mask);
      
  // we need to check to byteswap when we have cross platforms

  if ((!msb && lsb()) || (msb && !lsb())) {
    for (int i=0; i<s; i++) {
      unsigned int r = psColors_[i*3+2];
      unsigned int g = psColors_[i*3+1];
      unsigned int b = psColors_[i*3];
      unsigned int a = 0;
      a |= r << rs;
      a |= g << gs;
      a |= b << bs;
      memcpy(colors_+i*4, &a, 4);
    }
  }
  else {
    for (int i=0; i<s; i++) {
      unsigned int r = psColors_[i*3+2];
      unsigned int g = psColors_[i*3+1];
      unsigned int b = psColors_[i*3];
      unsigned int a = 0;
      a |= r << rs;
      a |= g << gs;
      a |= b << bs;

      unsigned char* rr = (unsigned char*)(&a);
      *(colors_+i*4) = *(rr+3);
      *(colors_+i*4+1) = *(rr+2);
      *(colors_+i*4+2) = *(rr+1);
      *(colors_+i*4+3) = *(rr);
    }
  }
}

ColorScaleTrueColor32::~ColorScaleTrueColor32()
{
  if (colors_)
    delete [] colors_;
}

LinearScaleTrueColor32::LinearScaleTrueColor32(int s, 
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count, 
					       unsigned long red_mask,
					       unsigned long green_mask,
					       unsigned long blue_mask, 
					       int msb) 
  : LinearScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, red_mask, green_mask, blue_mask, msb),
    ColorScale(s) {}

LogScaleTrueColor32::LogScaleTrueColor32(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells, int count,
					 double low, double high,
					 unsigned long red_mask,
					 unsigned long green_mask,
					 unsigned long blue_mask, 
					 int msb)
  : LogScale(s, indexCells, colorCells, count, low, high), 
    ColorScaleTrueColor32(s, red_mask, green_mask, blue_mask, msb), 
    ColorScale(s) {}

PowScaleTrueColor32::PowScaleTrueColor32(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells, int count,
					 double low, double high,
					 unsigned long red_mask,
					 unsigned long green_mask,
					 unsigned long blue_mask, 
					 int msb)
  : PowScale(s, indexCells, colorCells, count, low, high), 
    ColorScaleTrueColor32(s, red_mask, green_mask, blue_mask, msb), 
    ColorScale(s) {}

SqrtScaleTrueColor32::SqrtScaleTrueColor32(int s,
					   unsigned short* indexCells,
					   unsigned char* colorCells,
					   int count, 
					   unsigned long red_mask,
					   unsigned long green_mask,
					   unsigned long blue_mask, 
					   int msb)
  : SqrtScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, red_mask, green_mask, blue_mask, msb), 
    ColorScale(s) {}

SquaredScaleTrueColor32::SquaredScaleTrueColor32(int s,
						 unsigned short* indexCells,
						 unsigned char* colorCells,
						 int count, 
						 unsigned long red_mask,
						 unsigned long green_mask,
						 unsigned long blue_mask, 
						 int msb)
  : SquaredScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor32(s, red_mask, green_mask, blue_mask, msb), 
    ColorScale(s) {}

IISScaleTrueColor32::IISScaleTrueColor32(unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count, 
					 unsigned long red_mask,
					 unsigned long green_mask,
					 unsigned long blue_mask, 
					 int msb) 
  : IISScale(indexCells, colorCells, count), 
    ColorScaleTrueColor32(IISSIZE, red_mask, green_mask, blue_mask, msb),
    ColorScale(IISSIZE) {}

HistEquScaleTrueColor32::HistEquScaleTrueColor32(int s, 
						 unsigned short* indexCells,
						 unsigned char* colorCells,
						 int count, 
						 double* hist, int histsize,
						 unsigned long red_mask,unsigned long green_mask,unsigned long blue_mask, 
						 int msb) 
  : HistEquScale(s, indexCells, colorCells, count, hist, histsize), 
    ColorScaleTrueColor32(s, red_mask, green_mask, blue_mask, msb),
    ColorScale(s) {}


