// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "colorscaletrue8.h"
#include "util.h"

ColorScaleTrueColor8::ColorScaleTrueColor8(int s,
					   unsigned long red_mask,
					   unsigned long green_mask,
					   unsigned long blue_mask)
  : ColorScale(s)
{
  colors_ = new unsigned char[s];

  int rs,gs,bs;
  unsigned char rm = decodeMask((unsigned char)red_mask, &rs);
  unsigned char gm = decodeMask((unsigned char)green_mask, &gs);
  unsigned char bm = decodeMask((unsigned char)blue_mask, &bs);

  for (int i=0; i<s; i++)
    colors_[i] =
      ((psColors_[i*3]   & bm) >> bs) |  // blue
      ((psColors_[i*3+1] & gm) >> gs) |  // green
      ((psColors_[i*3+2] & rm) >> rs);   // red
}

ColorScaleTrueColor8::~ColorScaleTrueColor8()
{
  if (colors_)
    delete [] colors_;
}

LinearScaleTrueColor8::LinearScaleTrueColor8(int s, 
					     unsigned short* indexCells,
					     unsigned char* colorCells,
					     int count,
					     unsigned long red_mask,
					     unsigned long green_mask,
					     unsigned long blue_mask) 
  : LinearScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor8(s, red_mask, green_mask, blue_mask),
    ColorScale(s) {}

LogScaleTrueColor8::LogScaleTrueColor8(int s,
				       unsigned short* indexCells,
				       unsigned char* colorCells,
				       int count,
				       double low, double high,
				       unsigned long red_mask,
				       unsigned long green_mask,
				       unsigned long blue_mask) 
  : LogScale(s, indexCells, colorCells, count, low, high), 
    ColorScaleTrueColor8(s, red_mask, green_mask, blue_mask),
    ColorScale(s) {}

PowScaleTrueColor8::PowScaleTrueColor8(int s,
				       unsigned short* indexCells,
				       unsigned char* colorCells,
				       int count,
				       double low, double high,
				       unsigned long red_mask,
				       unsigned long green_mask,
				       unsigned long blue_mask) 
  : PowScale(s, indexCells, colorCells, count, low, high), 
    ColorScaleTrueColor8(s, red_mask, green_mask, blue_mask),
    ColorScale(s) {}

SqrtScaleTrueColor8::SqrtScaleTrueColor8(int s,
					 unsigned short* indexCells,
					 unsigned char* colorCells,
					 int count,
					 unsigned long red_mask,
					 unsigned long green_mask,
					 unsigned long blue_mask) 
  : SqrtScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor8(s, red_mask, green_mask, blue_mask),
    ColorScale(s) {}

SquaredScaleTrueColor8::SquaredScaleTrueColor8(int s,
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count,
					       unsigned long red_mask,
					       unsigned long green_mask,
					       unsigned long blue_mask) 
  : SquaredScale(s, indexCells, colorCells, count), 
    ColorScaleTrueColor8(s, red_mask, green_mask, blue_mask),
    ColorScale(s) {}

IISScaleTrueColor8::IISScaleTrueColor8(unsigned short* indexCells,
				       unsigned char* colorCells,
				       int count,
				       unsigned long red_mask,
				       unsigned long green_mask,
				       unsigned long blue_mask) 
  : IISScale(indexCells, colorCells, count), 
    ColorScaleTrueColor8(IISSIZE, red_mask, green_mask, blue_mask),
    ColorScale(IISSIZE) {}

HistEquScaleTrueColor8::HistEquScaleTrueColor8(int s, 
					       unsigned short* indexCells,
					       unsigned char* colorCells,
					       int count,
					       double* hist, int histsize,
					       unsigned long red_mask,
					       unsigned long green_mask,
					       unsigned long blue_mask) 
  : HistEquScale(s, indexCells, colorCells, count, hist, histsize), 
    ColorScaleTrueColor8(s, red_mask, green_mask, blue_mask),
    ColorScale(s) {}


