// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "fitsimage.h"
#include "rice.h"
#include "gzip.h"
#include "hcompress.h"
#include "plio.h"
#include "util.h"

void FitsImage::initCompress()
{
  int bitpix = fits_->getInteger("ZBITPIX",0);
  char* type = fits_->getString("ZCMPTYPE");
  if (!bitpix || !type)
    return ;

  if (compress_)
    delete compress_;
  compress_ = NULL;

  if (!strncmp(type,"RICE",4))
    switch (bitpix) {
    case 8: 
      compress_ = new FitsRicem<unsigned char>(fits_);
      break;
    case 16:
      compress_ = new FitsRicem<short>(fits_);
      break;
    case -16:
      compress_ = new FitsRicem<unsigned short>(fits_);
      break;
    case 32:
      compress_ = new FitsRicem<int>(fits_);
      break;
    case 64:
      compress_ = new FitsRicem<long long>(fits_);
      break;
    case -32:
      compress_ = new FitsRicem<float>(fits_);
      break;
    case -64:
      compress_ = new FitsRicem<double>(fits_);
      break;
    }
  else if (!strncmp(type,"GZIP",4))
    switch (bitpix) {
    case 8: 
      compress_ = new FitsGzipm<unsigned char>(fits_);
      break;
    case 16:
      compress_ = new FitsGzipm<short>(fits_);
      break;
    case -16:
      compress_ = new FitsGzipm<unsigned short>(fits_);
      break;
    case 32:
      compress_ = new FitsGzipm<int>(fits_);
      break;
    case 64:
      compress_ = new FitsGzipm<long long>(fits_);
      break;
    case -32:
      compress_ = new FitsGzipm<float>(fits_);
      break;
    case -64:
      compress_ = new FitsGzipm<double>(fits_);
      break;
    }
  else if (!strncmp(type,"PLIO",4))
    switch (bitpix) {
    case 8: 
      compress_ = new FitsPliom<unsigned char>(fits_);
      break;
    case 16:
      compress_ = new FitsPliom<short>(fits_);
      break;
    case -16:
      compress_ = new FitsPliom<unsigned short>(fits_);
      break;
    case 32:
      compress_ = new FitsPliom<int>(fits_);
      break;
    case 64:
      compress_ = new FitsPliom<long long>(fits_);
      break;
    case -32:
      compress_ = new FitsPliom<float>(fits_);
      break;
    case -64:
      compress_ = new FitsPliom<double>(fits_);
      break;
    }
  else if (!strncmp(type,"HCOMPRESS",4))
    switch (bitpix) {
    case 8: 
      compress_ = new FitsHcompressm<unsigned char>(fits_);
      break;
    case 16:
      compress_ = new FitsHcompressm<short>(fits_);
      break;
    case -16:
      compress_ = new FitsHcompressm<unsigned short>(fits_);
      break;
    case 32:
      compress_ = new FitsHcompressm<int>(fits_);
      break;
    case 64:
      compress_ = new FitsHcompressm<long long>(fits_);
      break;
    case -32:
      compress_ = new FitsHcompressm<float>(fits_);
      break;
    case -64:
      compress_ = new FitsHcompressm<double>(fits_);
      break;
    }

 return;
}
