// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "fitsmask.h"
#include "fitsimage.h"
#include "util.h"

FitsMask::FitsMask(FitsImage* fits, XColor* clr) 
{
  mask_ = fits;
  current_ = mask_;
  mptr_ = current_;
  color_ = clr;

  next_ = NULL;
  previous_ = NULL;
}

FitsMask::~FitsMask()
{
  FitsImage* ptr = mask_;
  while (ptr) {
    FitsImage* sptr = ptr->nextSlice();
    while (sptr) {
      FitsImage* stmp = sptr->nextSlice();
      delete sptr;
      sptr = stmp;
    }

    FitsImage* tmp = ptr->nextMosaic();
    delete ptr;
    ptr = tmp;
  }
}

void FitsMask::nextMosaic() {
  if (mptr_) 
    mptr_ = mptr_->nextMosaic();
}

void FitsMask::nextSlice() {
  if (current_) 
    current_ = current_->nextSlice();
}

void FitsMask::loadFinish()
{
  FitsImage* ptr = mask_;
  while (ptr && ptr->nextMosaic()) {
    int jj=0;
    FitsImage* sptr = ptr;
    while (sptr) {
      if (sptr->nextMosaic() == NULL) {
	// ok, let's figure out next inline
	FitsImage* mptr = ptr->nextMosaic();
	for (int nn=0; nn<jj; nn++)
	  mptr = mptr->nextSlice();
	sptr->setNextMosaic(mptr);
      }
      jj++;
      sptr = sptr->nextSlice();
    }
    ptr = ptr->nextMosaic();
  }
}

