// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __frame_h__
#define __frame_h__

#include "framebase.h"
#include "colorscale.h"

// Frame

class Frame : virtual public FrameBase {
 protected:
  FitsImage* fits;           // list of fits images

  int mosaicCount;             // count of fits images
  int sliceCount;             // count of fits images

  int cmapID;                // current colormap id
  float bias;                // current colormap bias
  float contrast;            // current colormap contrast

  int colorCount;            // number of dynamic colors
  ColorScale* colorScale;    // current color scale
  unsigned char* colorCells; // current color values
  unsigned short* indexCells; // current color indices

  FrScale frScale;           // scale parameters
  MosaicType mosaicType;
  CoordSystem mosaicSystem;

  Contour* contour;

 private:
  void buildXImage(XImage*, int, int, int, int,
		   double* (FitsImage::*)());

  void psLevel1(PSColorSpace, int, int, float);

  void psLevel1Single(PSColorSpace, int, int, float);
  void psLevel1Mosaic(PSColorSpace, int, int, float);

  void reset();
  void setCurrentFits(FitsImage* f) {currentFits = f;}
  void setKeyFits() {};
  void setBinCursor(const Vector&);
  void updateBin(const Matrix&);

  BBox imageBBox();
  void alignWCS();
  void alignWCS(FitsImage*);

  void updateMatrices();
  void updateMagnifierMatrices(Matrix&);
  int doRender() {return currentFits ? 1 : 0;}
  int isIIS();
  void unload();
  void unloadFits();

 protected:
  int validColorScale() {return colorScale ? 1 : 0;}
  void unloadAllFits() {unloadFits();}
  void updateColorCells(unsigned short*, unsigned char*, int);

  virtual void fillXImageSingle(XImage*, int, int, int, int,
				double* (FitsImage::*)()) =0;
  virtual void fillXImageMosaic(XImage*, int, int, int, int,
				double* (FitsImage::*)()) =0;

 public:
  Frame(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Frame();

  // Fits Commands

  void sliceCmd(int);
  void getWCSAlignCmd();

  // Colormap Commands

  void getColormapCmd();
  void getTypeCmd();

  // IIS Commands

  void iisCmd(int, int);
  void iisEraseCmd();
  void iisGetCmd(char*, int, int, int, int);
  void iisSetCmd(const char*, int, int, int, int);
  void iisWCSCmd(const Matrix&, const Vector&, int);

  // RGB Commands

  void getRGBChannelCmd();
  void getRGBViewCmd();
  void getRGBSystemCmd();
};

#endif
