// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "framepseudocolor8.h"
#include "colorscalepseudo8.h"
#include "util.h"

// Tk Canvas Widget Function Declarations

int FramePseudoColor8CreateProc(Tcl_Interp*, Tk_Canvas, Tk_Item*, int, 
				Tcl_Obj *const []);

// FramePseudoColor8 Specs

static Tk_CustomOption tagsOption = {
  Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
};

static Tk_ConfigSpec framePseudoColor8Specs[] = {

  {TK_CONFIG_STRING, "-command", NULL, NULL, "frame",
   Tk_Offset(WidgetOptions, cmdName), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-x", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, x), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-y", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, y), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-width", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, width), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-height", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, height), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_ANCHOR, "-anchor", NULL, NULL, "nw",
   Tk_Offset(WidgetOptions, anchor), 0, NULL},
  {TK_CONFIG_CUSTOM, "-tags", NULL, NULL, NULL,
   0, TK_CONFIG_NULL_OK, &tagsOption},

  {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL},
};

// Tk Static Structure

static Tk_ItemType framePseudoColor8Type = {
  "framepseudocolor8",          // name
  sizeof(WidgetOptions),        // item size
  FramePseudoColor8CreateProc,  // configProc
  framePseudoColor8Specs,       // configSpecs
  WidgetConfigProc,             // configProc
  WidgetCoordProc,              // coordProc
  WidgetDeleteProc,             // deleteProc
  WidgetDisplayProc,            // displayProc
  0,                            // alwaysRedraw
  WidgetPointProc,              // pointProc
  WidgetAreaProc,               // areaProc
  WidgetPostscriptProc,         // postscriptProc
  WidgetScaleProc,              // scaleProc
  WidgetTranslateProc,          // translateProc
  (Tk_ItemIndexProc*)NULL,      // indexProc
  WidgetICursorProc,            // icursorProc
  (Tk_ItemSelectionProc*)NULL,  // selectionProc
  (Tk_ItemInsertProc*)NULL,     // insertProc
  (Tk_ItemDCharsProc*)NULL,     // dCharsProc
  (Tk_ItemType*)NULL            // nextPtr
};

// Non-Member Functions

int FramePseudoColor8_Init(Tcl_Interp* interp)
{
  Tk_CreateItemType(&framePseudoColor8Type);
  return TCL_OK;
}

int FramePseudoColor8CreateProc(Tcl_Interp* interp, Tk_Canvas canvas, 
				Tk_Item* item, int argc, Tcl_Obj *const argv[])
{
  FramePseudoColor8* frame = new FramePseudoColor8(interp, canvas, item);

  // and set default configuration

  if (frame->configure(argc, (const char**)argv, 0) != TCL_OK) {
    delete frame;
    Tcl_AppendResult(interp, " error occured while creating frame.", NULL);
    return TCL_ERROR;
  }

  return TCL_OK;
}

// FramePseudoColor8 Member Functions

FramePseudoColor8::FramePseudoColor8(Tcl_Interp* i, Tk_Canvas c, Tk_Item* item)
  : FrameBase(i,c,item), FramePseudoColor(i,c,item)
{
  configSpecs = framePseudoColor8Specs;  // frame configure options
}

FramePseudoColor8::~FramePseudoColor8()
{
  // we must do this at this level, because updateColorScale is called
  unloadAllFits();
}

void FramePseudoColor8::updateColorScale()
{
  // we need colors before we can construct a scale

  if (!indexCells || !colorCells)
    return;

  if (colorScale)
    delete colorScale;

  switch (frScale.colorScaleType()) {
  case FrScale::LINEARSCALE:
    colorScale =
      new LinearScalePseudoColor8(colorCount, indexCells, colorCells, 
				  colorCount);
    break;
  case FrScale::LOGSCALE:
    colorScale = new LogScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
					  colorCount, 
					  currentScale->low(),
					  currentScale->high());
    break;
  case FrScale::POWSCALE:
    colorScale = new PowScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
					  colorCount, 
					  currentScale->low(),
					  currentScale->high());
    break;
  case FrScale::SQRTSCALE:
    colorScale = 
      new SqrtScalePseudoColor8(SCALESIZE, indexCells, colorCells, colorCount);
    break;
  case FrScale::SQUAREDSCALE:
    colorScale =
      new SquaredScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
				   colorCount);
    break;
  case FrScale::IISSCALE:
    colorScale =
      new IISScalePseudoColor8(this,indexCells, colorCells, colorCount);
    break;
  case FrScale::HISTEQUSCALE:
    calcHistEqu();
    colorScale =
      new HistEquScalePseudoColor8(SCALESIZE, indexCells, colorCells, 
				   colorCount, currentScale->histequ(),
				   HISTEQUSIZE);
    break;
  }
}


