// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include "frametruecolor16.h"
#include "colorscaletrue16.h"

// Tk Canvas Widget Function Declarations

int FrameTrueColor16CreateProc(Tcl_Interp*, Tk_Canvas, Tk_Item*, int, Tcl_Obj *const []);

// FrameTrueColor16 Specs

static Tk_CustomOption tagsOption = {
  Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
};

static Tk_ConfigSpec frameTrueColor16Specs[] = {

  {TK_CONFIG_STRING, "-command", NULL, NULL, "frame",
   Tk_Offset(WidgetOptions, cmdName), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-x", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, x), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-y", NULL, NULL, "1",
   Tk_Offset(WidgetOptions, y), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-width", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, width), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_INT, "-height", NULL, NULL, "512",
   Tk_Offset(WidgetOptions, height), TK_CONFIG_OPTION_SPECIFIED, NULL},
  {TK_CONFIG_ANCHOR, "-anchor", NULL, NULL, "nw",
   Tk_Offset(WidgetOptions, anchor), 0, NULL},
  {TK_CONFIG_CUSTOM, "-tags", NULL, NULL, NULL,
   0, TK_CONFIG_NULL_OK, &tagsOption},

  {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL},
};

// Tk Static Structure

static Tk_ItemType frameTrueColor16Type = {
  "frametruecolor16",           // name
  sizeof(WidgetOptions),        // item size
  FrameTrueColor16CreateProc,   // configProc
  frameTrueColor16Specs,        // configSpecs
  WidgetConfigProc,             // configProc
  WidgetCoordProc,              // coordProc
  WidgetDeleteProc,             // deleteProc
  WidgetDisplayProc,            // displayProc
  0,                            // alwaysRedraw
  WidgetPointProc,              // pointProc
  WidgetAreaProc,               // areaProc
  WidgetPostscriptProc,         // postscriptProc
  WidgetScaleProc,              // scaleProc
  WidgetTranslateProc,          // translateProc
  (Tk_ItemIndexProc*)NULL,      // indexProc
  WidgetICursorProc,            // icursorProc
  (Tk_ItemSelectionProc*)NULL,  // selectionProc
  (Tk_ItemInsertProc*)NULL,     // insertProc
  (Tk_ItemDCharsProc*)NULL,     // dCharsProc
  (Tk_ItemType*)NULL            // nextPtr
};

// Non-Member Functions

int FrameTrueColor16_Init(Tcl_Interp* interp)
{
  Tk_CreateItemType(&frameTrueColor16Type);
  return TCL_OK;
}

int FrameTrueColor16CreateProc(Tcl_Interp* interp, Tk_Canvas canvas, 
			       Tk_Item* item, int argc, Tcl_Obj *const argv[])
{
  FrameTrueColor16* frame = new FrameTrueColor16(interp, canvas, item);

  // and set default configuration

  if (frame->configure(argc, (const char**)argv, 0) != TCL_OK) {
    delete frame;
    Tcl_AppendResult(interp, " error occured while creating frame.", NULL);
    return TCL_ERROR;
  }

  return TCL_OK;
}

// FrameTrueColor16 Member Functions

FrameTrueColor16::FrameTrueColor16(Tcl_Interp* i, Tk_Canvas c, Tk_Item* item)
  : FrameBase(i,c,item), FrameTrueColor(i,c,item)
{
  configSpecs = frameTrueColor16Specs;  // frame configure options
}

FrameTrueColor16::~FrameTrueColor16()
{
  // we must do this at this level, because updateColorScale is called
  unloadAllFits();
}

void FrameTrueColor16::updateColorScale()
{
  // we need colors before we can construct a scale

  if (!indexCells || !colorCells)
    return;

  // since we need to know about bytes per pixel and byte order, 
  // we may need to wait til we have an XImage available and try again

  if (!baseXImage)
    return;

  encodeTrueColor(bgColor, bgTrueColor_);
  encodeTrueColor(nanColor, nanTrueColor_);

  if (colorScale)
    delete colorScale;

#if !(_WIN32 || _MACOSX)
  unsigned long red_mask = visual->red_mask;
  unsigned long green_mask = visual->green_mask;
  unsigned long blue_mask = visual->blue_mask;
#endif
#if _MACOSX
  // tk visual masks are bogus
  unsigned long red_mask = baseXImage->red_mask;
  unsigned long green_mask = baseXImage->green_mask;
  unsigned long blue_mask = baseXImage->blue_mask;
#endif
#if _WIN32
  // windows masks are plain wrong
  unsigned long red_mask = 0x00007C00;
  unsigned long green_mask = 0x000003E0;
  unsigned long blue_mask = 0x0000001F;
#endif

  switch (currentScale->colorScaleType()) {
  case FrScale::LINEARSCALE:
    colorScale =
      new LinearScaleTrueColor16(colorCount, indexCells, colorCells, 
				 colorCount, 
				 red_mask, green_mask, blue_mask,
				 baseXImage->byte_order);
    break;
  case FrScale::LOGSCALE:
    colorScale =
      new LogScaleTrueColor16(SCALESIZE, indexCells, colorCells, colorCount,
			      currentScale->low(), currentScale->high(),
			      red_mask, green_mask, blue_mask, 
			      baseXImage->byte_order);
    break;
  case FrScale::POWSCALE:
    colorScale =
      new PowScaleTrueColor16(SCALESIZE, indexCells, colorCells, colorCount,
			      currentScale->low(), currentScale->high(),
			      red_mask, green_mask, blue_mask, 
			      baseXImage->byte_order);
    break;
  case FrScale::SQRTSCALE:
    colorScale = 
      new SqrtScaleTrueColor16(SCALESIZE, indexCells, colorCells, colorCount,
			       red_mask, green_mask, blue_mask, 
			       baseXImage->byte_order);
    break;
  case FrScale::SQUAREDSCALE:
    colorScale =
      new SquaredScaleTrueColor16(SCALESIZE, indexCells, colorCells, 
				  colorCount, 
				  red_mask, green_mask, blue_mask, 
				  baseXImage->byte_order);
    break;
  case FrScale::IISSCALE:
    colorScale =
      new IISScaleTrueColor16(indexCells, colorCells, colorCount, 
			      red_mask, green_mask, blue_mask, 
			      baseXImage->byte_order);
    break;
  case FrScale::HISTEQUSCALE:
    calcHistEqu();
    colorScale =
      new HistEquScaleTrueColor16(SCALESIZE, indexCells, colorCells, 
				  colorCount, currentScale->histequ(),
				  HISTEQUSIZE, 
				  red_mask, green_mask, blue_mask, 
				  baseXImage->byte_order);
    break;
  }
}
