// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __panner_h__
#define __panner_h__

#include "widget.h"

class Panner : public Widget {
private:
  Pixmap thumbnail;          // current frame thumbnail pixmap
  Vector bb[4];              // current view
  int highLite;              // flag to highlite bbox
  int panning;               // flag for panning status
  Vector panStart;           // initial location of panning operation
  Rotate imageCompass;       // compass orientation matrix
  Vector wcsNorth;           // wcs north unit vector
  Vector wcsEast;            // wcs east unit vector
  int useBBox;               // flag to use BBox
  int useImageCompass;       // flag to use image compass
  int useWCSCompass;         // flag to use wcs compass
  int validWCSCompass;       // flag to indicate wcs data is valid
  int useFont;               // flag to use fonts
  int needsUpdate;           // flag to indicate refresh needed

  Tk_Font font;              // Tk font
  Tk_FontMetrics metric;     // Tk font metric
				  
  GC bboxGC;                 // bbox gc
  GC compassGC;              // compass gc

  void update();             // use existing pixmap, just update
  void invalidPixmap();      // new pixmap, then update

protected:
  virtual void clearPixmap() =0;  // clear pixmap

private:
  int updatePixmap(const BBox&); // renders image/graphics into pixmap
  void renderBBox();
  int isInBBox(const Vector&);
  void renderImageCompass();
  void renderWCSCompass();
  void renderArm(int, const Vector&, Rotate&, char*, int);
  void updateGCs();

public:
  Panner(Tcl_Interp*, Tk_Canvas, Tk_Item*);
  virtual ~Panner();

  int parse(istringstream&);            // parse subcommands

  // SubCommandFunctions

  void getBBoxCmd();
  void getSizeCmd();
  void highLiteCmd(int);
  void highLiteCmd(const Vector&);
  void panToCmd(const Vector&);
  void panBeginCmd(const Vector&);
  void panMotionCmd(const Vector&);
  void panEndCmd(const Vector&);
  void setImageCompassCmd(int);
  void setWCSCompassCmd(int);
  void setBBoxCmd(int);
  void updateCmd(Pixmap);
  void updateBBoxCmd(const Vector&, const Vector&, 
		     const Vector&, const Vector&);
  void updateImageCompassCmd(Rotate);
  void updateWCSCompassCmd();
  void updateWCSCompassCmd(const Vector&, const Vector&);
  void warpCmd(const Vector&);
};

#endif

