// Copyright (C) 1999-2008
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#ifndef __util_h__
#define __util_h__

#include <iostream>
using namespace std;

#include "vector.h"

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

#define XImageData(x) x->data

#ifdef _WIN32
#include <xxlib.h>
#include <win32lib.h>
#define XGetImage XXGetImage
#define XPutImage XXPutImage
#define XQueryPointer XXQueryPointer
#define XWarpPointer XXWarpPointer
#endif

#ifdef _MACOSX
#undef XImageData
#include <xxlib.h>
#include <macosxlib.h>
#define XGetImage XXGetImage
#define XQueryPointer XXQueryPointer
#define XWarpPointer XXWarpPointer
#endif

extern int lsb();

extern void swap2(char* src, char* dest);
extern void swap4(char* src, char* dest);
extern void swap8(char* src, char* dest);

extern char* dupstr(const char*);
extern char* toLower(const char*);
extern char* toUpper(const char*);

extern double degToRad(double);
extern double radToDeg(double);

extern double dmsToDegree(int, int, int, double);
extern double parseSEXStr(const char*);
extern double parseHMSStr(const char*);
extern double parseDMSStr(const char*);

extern unsigned char decodeMask(unsigned char, int*);
extern unsigned short decodeMask(unsigned short, int*);
extern int decodeMask(unsigned long);

extern double RGB2Gray(double, double, double);
extern unsigned char RGB2Gray(unsigned char, unsigned char, unsigned char);
extern void RGB2CMYK(unsigned char, unsigned char, unsigned char,
		     unsigned char*, unsigned char*, unsigned char*, 
		     unsigned char*);
extern void RGB2CMYK(unsigned short, unsigned short, unsigned short,
		     unsigned short*, unsigned short*, unsigned short*, 
		     unsigned short*);
extern ostream& psColorGray(XColor*, ostream&);
extern ostream& psColorRGB(XColor*, ostream&);
extern ostream& psColorCMYK(XColor*, ostream&);

extern char* psQuote(const char*);

extern int fCompare(const void*, const void*);
extern int dCompare(const void*, const void*);

extern void CheckScaleLog(double*, double*);

extern Vector mapLen(const Vector& v, const Matrix& mx);

extern "C" {
extern void conputs(FILE *fd, char *format, ...);
}

#endif
